@@ -99,7 +99,7 @@ func (c *ProjectInspectCommand) FormatProject(projectTarget string) error {
 		workspaceNames = append(workspaceNames, ws.Workspace.Workspace)
 	}
 
-	var gitUrl, gitRef, gitPath string
+	var gitUrl, gitRef, gitPath, remoteDesc string
 	dataSource := "Local" // if unset, assume local
 	if project.DataSource != nil {
 		switch ds := project.DataSource.Source.(type) {
@@ -111,6 +111,9 @@ func (c *ProjectInspectCommand) FormatProject(projectTarget string) error {
 			gitUrl = ds.Git.Url
 			gitRef = ds.Git.Ref
 			gitPath = ds.Git.Path
+		case *pb.Job_DataSource_Remote:
+			dataSource = "Remote"
+			remoteDesc = ds.Remote.Description
 		}
 	}
 
@@ -159,6 +162,9 @@ func (c *ProjectInspectCommand) FormatProject(projectTarget string) error {
 		{
 			Name: "Git Path", Value: gitPath,
 		},
+		{
+			Name: "Remote Info", Value: remoteDesc,
+		},
 		{
 			Name: "Data Source Poll Enabled", Value: strconv.FormatBool(datasourcePollEnabled),
 		},
