@@ -24,6 +24,19 @@ func (r *Runner) executeDeployOp(
 		panic("operation not expected type")
 	}
 
+	if op.Deploy.Artifact == nil {
+		// get latest artifact if none set
+		push, err := r.client.GetLatestPushedArtifact(ctx, &pb.GetLatestPushedArtifactRequest{
+			Application: job.Application,
+			Workspace:   job.Workspace,
+		})
+		if err != nil {
+			return nil, err
+		}
+
+		op.Deploy.Artifact = push
+	}
+
 	deploymentResult, err := app.Deploy(ctx, op.Deploy.Artifact)
 	if err != nil {
 		return nil, err
