@@ -402,6 +402,25 @@ func (s *State) JobList(
 			}
 		}
 
+		if req.Pipeline != nil {
+			if job.Pipeline == nil {
+				continue
+			}
+			// check whether the job is a match for pipeline name or id, and run sequence (if specified).
+			if req.Pipeline.RunSequence != 0 && (req.Pipeline.RunSequence != job.Pipeline.RunSequence) {
+				continue
+			}
+			if req.Pipeline.PipelineName != "" && (req.Pipeline.PipelineName != job.Pipeline.PipelineName) {
+				continue
+			}
+			if req.Pipeline.PipelineId != "" && (req.Pipeline.PipelineId != job.Pipeline.PipelineId) {
+				continue
+			}
+			if (req.Pipeline.PipelineId == job.Pipeline.PipelineId || req.Pipeline.PipelineName == job.Pipeline.PipelineName) && req.Pipeline.RunSequence != job.Pipeline.RunSequence {
+				continue
+			}
+		}
+
 		result = append(result, job)
 	}
 
@@ -669,9 +688,9 @@ RETRY_ASSIGN:
 // JobAck acknowledges that a job has been accepted or rejected by the runner.
 // If ack is false, then this will move the job back to the queued state
 // and be eligible for assignment.
-// Additionally, if a job is associated with an on-demand runner task, this
-// func will progress the Tasks state machine depending on which job has
-// currently been acked.
+// Additionally, if a job is associated with an on-demand runner task and/or pipeline,
+// this func will progress the Tasks and Pipeline state machines depending on which job
+// has currently been acked.
 func (s *State) JobAck(id string, ack bool) (*serverstate.Job, error) {
 	txn := s.inmem.Txn(true)
 	defer txn.Abort()
@@ -731,8 +750,7 @@ func (s *State) JobAck(id string, ack bool) (*serverstate.Job, error) {
 	job.StateTimer = time.AfterFunc(serverstate.JobHeartbeatTimeout, func() {
 		s.log.Info("canceling job due to heartbeat timeout", "job", job.Id)
 		// Force cancel
-		err := s.JobCancel(job.Id, true)
-		if err != nil {
+		if err := s.JobCancel(job.Id, true); err != nil {
 			s.log.Error("error canceling job due to heartbeat failure", "error", err, "job", job.Id)
 		}
 	})
@@ -758,7 +776,10 @@ func (s *State) JobAck(id string, ack bool) (*serverstate.Job, error) {
 		s.log.Error("error updating task state", "error", err, "job", job.Id)
 		return nil, err
 	}
-
+	if err := s.pipelineAck(job.Id); err != nil {
+		s.log.Error("error updating pipeline state", "error", err, "job", job.Id)
+		return nil, err
+	}
 	return job.Job(result), nil
 }
 
@@ -771,6 +792,17 @@ func (s *State) JobUpdateRef(id string, ref *pb.Job_DataSource_Ref) error {
 	})
 }
 
+// JobUpdateExpiry will update the jobs expiration time with the new value. This
+// method is used when a runner has accepted a job and peeks at its runner
+// config. By this point, we know the runner has accepted a job to work on,
+// so the runner should handle the job soon.
+func (s *State) JobUpdateExpiry(id string, newExpire *timestamppb.Timestamp) error {
+	return s.JobUpdate(id, func(jobpb *pb.Job) error {
+		jobpb.ExpireTime = newExpire
+		return nil
+	})
+}
+
 // JobUpdate calls the given callback to update fields on the job data.
 // The callback is called in the context of a database write lock so it
 // should NOT compute anything and should be fast. The callback can return
@@ -877,6 +909,13 @@ func (s *State) JobComplete(id string, result *pb.Job_Result, cerr error) error
 		return err
 	}
 
+	// If the job is part of a pipeline, update the pipeline state machine
+	// to mark the pipeline as complete
+	if err := s.pipelineComplete(job.Id); err != nil {
+		s.log.Error("error updating pipeline state for complete", "error", err, "job", job.Id)
+		return err
+	}
+
 	return nil
 }
 
@@ -902,6 +941,11 @@ func (s *State) JobCancel(id string, force bool) error {
 	}
 
 	txn.Commit()
+
+	if err := s.pipelineCancel(id); err != nil {
+		return err
+	}
+
 	return nil
 }
 
@@ -1296,6 +1340,11 @@ func (s *State) jobCreate(dbTxn *bolt.Tx, memTxn *memdb.Txn, jobpb *pb.Job) erro
 
 		// Go through and ensure that each exists.
 		for _, id := range jobpb.DependsOn {
+			if id == "" {
+				return status.Errorf(codes.FailedPrecondition,
+					"job %q has an empty string id for depends_on", jobpb.Id)
+			}
+
 			dependsMap[id] = struct{}{}
 
 			_, err := s.jobById(dbTxn, id)
@@ -1598,7 +1647,7 @@ func (s *State) jobCandidateAny(
 	return nil, nil
 }
 
-// taskAck looks to see if the referenced job id has a Task ref associated with it,
+// taskAck checks if the referenced job id has a Task ref associated with it,
 // and if so, Ack the specific job inside the Task job triple to progress the
 // Task state machine.
 func (s *State) taskAck(jobId string) error {
@@ -1698,6 +1747,112 @@ func (s *State) taskComplete(jobId string) error {
 	return nil
 }
 
+// pipelineComplete will look up the referenced job to see if it has a PipelineStep ref
+// associated with it, and if so, progress the pipelineRun state machine.
+func (s *State) pipelineComplete(jobId string) error {
+	// grab the job
+	job, err := s.JobById(jobId, nil)
+	if err != nil {
+		s.log.Error("error getting job by id", "job", jobId, "err", err)
+		return err
+	} else if job.Pipeline == nil {
+		s.log.Trace("job is not part of a pipeline", "job", jobId)
+		return nil
+	}
+	// grab the pipeline run
+	run, err := s.PipelineRunGetByJobId(jobId)
+	if run == nil {
+		return nil
+	}
+	if err != nil {
+		s.log.Error("failed to retrieve pipeline to complete", "job", job.Id)
+		return err
+	}
+
+	if job.State == pb.Job_ERROR {
+		run.State = pb.PipelineRun_ERROR
+	} else if job.State == pb.Job_SUCCESS {
+		// If job Id matches last job queued by pipeline.
+		// We will have to change this in the future when pipeline steps run in parallel,
+		// and the last job queued may not be the last job to complete in the pipeline
+		// TODO:XX figure out how ^
+		if job.Id == run.Jobs[len(run.Jobs)-1].Id {
+			run.State = pb.PipelineRun_SUCCESS
+			s.log.Trace("pipeline run is complete", "job", job.Id, "pipeline", job.Pipeline.PipelineId, "run", run.Sequence)
+		}
+	}
+
+	// PipelineRunPut the new state
+	if err = s.PipelineRunPut(run); err != nil {
+		s.log.Error("failed to complete pipeline run", "job", job.Id, "pipeline", job.Pipeline.PipelineId, "run", job.Pipeline.RunSequence)
+		return err
+	}
+
+	return nil
+}
+
+// pipelineAck checks if the referenced job id has a PipelineStep ref associated with it,
+// and if so, progress the PipelineRun state machine.
+func (s *State) pipelineAck(jobId string) error {
+	// grab pipeline run that triggered the job based on the PipelineTask Ref
+	run, err := s.PipelineRunGetByJobId(jobId)
+	if run == nil {
+		return nil
+	}
+	if err != nil {
+		s.log.Error("failed to retrieve pipeline to complete", "job", jobId)
+		return err
+	}
+
+	// Update the new pipeline run state if it's not already running
+	if run.State != pb.PipelineRun_RUNNING {
+		run.State = pb.PipelineRun_RUNNING
+	}
+	s.log.Trace("pipeline is running", "job", jobId, "pipeline", run.Pipeline, "run", run.Sequence)
+	if err := s.PipelineRunPut(run); err != nil {
+		s.log.Error("failed to ack pipeline run state", "job", jobId, "pipeline", run.Pipeline, "run", run.Sequence)
+		return err
+	}
+
+	return nil
+}
+
+// pipelineCancel will look up the referenced job to see if it has a PipelineStep ref
+// associated with it, and if so, progress the pipelineRun state machine.
+func (s *State) pipelineCancel(jobId string) error {
+	job, err := s.JobById(jobId, nil)
+	if err != nil {
+		s.log.Error("error getting job by id", "job", jobId, "err", err)
+		return err
+	} else if job.Pipeline == nil {
+		s.log.Trace("job is not part of a pipeline", "job", jobId)
+		return nil
+	}
+
+	run, err := s.PipelineRunGetByJobId(jobId)
+	if run == nil {
+		return nil
+	}
+	if err != nil {
+		s.log.Error("failed to retrieve pipeline to complete", "job", job.Id)
+		return err
+	}
+
+	if job.State == pb.Job_SUCCESS {
+		return nil
+	} else {
+		run.State = pb.PipelineRun_CANCELLED
+		s.log.Trace("pipeline run cancelled", "job", job.Id, "pipeline", job.Pipeline.PipelineId, "run", run.Sequence)
+	}
+	// PipelineRunPut the new state
+	if err := s.PipelineRunPut(run); err != nil {
+		s.log.Error("failed to cancel pipeline run", "job", job.Id, "pipeline", job.Pipeline.PipelineId, "run", job.Pipeline.RunSequence)
+		return err
+	}
+
+	return nil
+}
+
 // Copy should be called prior to any modifications to an existing jobIndex.
 func (idx *jobIndex) Copy() *jobIndex {
 	// A shallow copy is good enough since we only modify top-level fields.
