@@ -2,6 +2,8 @@ package singleprocess
 
 import (
 	"context"
+	"github.com/hashicorp/go-hclog"
+	"github.com/hashicorp/waypoint/pkg/server/hcerr"
 
 	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/status"
@@ -20,7 +22,7 @@ func (s *Service) SetServerConfig(
 	}
 
 	if err := s.state(ctx).ServerConfigSet(req.Config); err != nil {
-		return nil, err
+		return nil, hcerr.Externalize(hclog.FromContext(ctx), err, "failed to set server config", "platform", req.Config.Platform)
 	}
 
 	return &empty.Empty{}, nil
@@ -32,7 +34,7 @@ func (s *Service) GetServerConfig(
 ) (*pb.GetServerConfigResponse, error) {
 	cfg, err := s.state(ctx).ServerConfigGet()
 	if err != nil {
-		return nil, err
+		return nil, hcerr.Externalize(hclog.FromContext(ctx), err, "failed to get server config")
 	}
 
 	return &pb.GetServerConfigResponse{Config: cfg}, nil
